/*global require,exports,describe,it,expect */
var Montage = require("montage").Montage,
    TestPageLoader = require("montage-testing/testpageloader").TestPageLoader,
    BOOTSTRAPPING = 0,
    LOADING = 1,
    LOADED = 2;

TestPageLoader.queueTest("loader/loader-test", function (testPage) {
    var test;

    beforeEach(function () {
        test = testPage.test;
    });

    describe("ui/loader/loader-spec", function () {
        // These tests need to run in order, also, they're not deterministic
        // we don't have enough mocks to drive the loader yet.
        // These tests should be consider sanity tests.

        it("should be in the BOOTSTRAPPING stage or after", function (done) {
            var loader = test.templateObjects.owner;
            setTimeout(function () {
                expect(loader.currentStage).not.toBe(BOOTSTRAPPING);
                done();
            }, "BOOTSTRAPPING is over", 2000);
        });

        it("should be in the LOADING stage or after", function (done) {
            var loader = test.templateObjects.owner;

            setTimeout(function () {
                expect(loader.currentStage).not.toBe(LOADING);
                done();
            }, "LOADING is over", 2000);
        });

        it("should be in the LOADED stage", function () {
            var loader = test.templateObjects.owner;

            expect(loader.currentStage).toBe(LOADED);
        });

        // After this point the Loader has finished its work.
        it("should load the main component", function () {
            var loader = test.templateObjects.owner,
                main = loader._mainComponent;

            expect(main.element.textContent).toBe(main.text);
        });

        it("should not be part of the component tree", function () {
            var loader = test.templateObjects.owner;

            expect(loader.parentComponent).toBeFalsy();
            expect(loader.childComponents.length).toBe(0);
        });

        it("should replace itself in the component tree with the main component", function () {
            var loader = test.templateObjects.owner,
                main = loader._mainComponent;

            expect(main.parentComponent).toBe(main.rootComponent);
        });
    });
});
